@@ -10,17 +10,17 @@ import os
 from io import StringIO
 from typing import Sequence, Type
 from unittest import skipIf
+
 from zope.interface import Interface
 
 from twisted import plugin
-from twisted.trial.unittest import TestCase
-from twisted.cred import credentials, checkers, error, strcred
-from twisted.plugins import cred_file, cred_anonymous, cred_unix
+from twisted.cred import checkers, credentials, error, strcred
+from twisted.plugins import cred_anonymous, cred_file, cred_unix
 from twisted.python import usage
-from twisted.python.filepath import FilePath
 from twisted.python.fakepwd import UserDatabase
+from twisted.python.filepath import FilePath
 from twisted.python.reflect import requireModule
-
+from twisted.trial.unittest import TestCase
 
 crypt = requireModule("crypt")
 pwd = requireModule("pwd")
@@ -417,6 +417,7 @@ class FileDBCheckerTests(TestCase):
         self.assertIn(cred_file.invalidFileWarning, newOutput.getvalue())
 
 
+@skipIf(not pwd, "Required module not available: pwd")
 @skipIf(not requireModule("cryptography"), "cryptography is not available")
 @skipIf(not requireModule("pyasn1"), "pyasn1 is not available")
 class SSHCheckerTests(TestCase):
