@@ -7,34 +7,31 @@ Very basic functionality for a Reactor implementation.
 """
 
 
-from abc import ABC, abstractmethod
 import builtins
-from heapq import heappush, heappop, heapify
 import socket  # needed only for sync-dns
-import sys
+import warnings
+from abc import ABC, abstractmethod
+from heapq import heapify, heappop, heappush
 from traceback import format_stack
 from types import FrameType
 from typing import (
+    TYPE_CHECKING,
     Any,
-    AnyStr,
     Callable,
     Dict,
     List,
-    Mapping,
     NewType,
     Optional,
     Sequence,
     Set,
     Tuple,
-    TYPE_CHECKING,
     Union,
     cast,
 )
-import warnings
 
-from zope.interface import implementer, classImplements
+from zope.interface import classImplements, implementer
 
-from twisted.internet import fdesc, main, error, abstract, defer, threads
+from twisted.internet import abstract, defer, error, fdesc, main, threads
 from twisted.internet._resolver import (
     ComplexResolverSimplifier as _ComplexResolverSimplifier,
     GAIResolver as _GAIResolver,
@@ -42,7 +39,6 @@ from twisted.internet._resolver import (
 )
 from twisted.internet.defer import Deferred, DeferredList
 from twisted.internet.interfaces import (
-    _ISupportsExitSignalCapturing,
     IAddress,
     IConnector,
     IDelayedCall,
@@ -56,11 +52,12 @@ from twisted.internet.interfaces import (
     IReadDescriptor,
     IResolverSimple,
     IWriteDescriptor,
+    _ISupportsExitSignalCapturing,
 )
 from twisted.internet.protocol import ClientFactory
 from twisted.python import log, reflect
 from twisted.python.failure import Failure
-from twisted.python.runtime import seconds as runtimeSeconds, platform
+from twisted.python.runtime import platform, seconds as runtimeSeconds
 
 if TYPE_CHECKING:
     from twisted.internet.tcp import Client
@@ -81,7 +78,10 @@ class DelayedCall:
     # enable .debug to record creator call stack, and it will be logged if
     # an exception occurs while the function is being run
     debug = False
-    _repr = None  # type: Optional[str]
+    _repr: Optional[str] = None
+
+    # In debug mode, the call stack at the time of instantiation.
+    creator: Optional[Sequence[str]] = None
 
     def __init__(
         self,
@@ -237,13 +237,14 @@ class DelayedCall:
         if hasattr(self, "func"):
             # This code should be replaced by a utility function in reflect;
             # see ticket #6066:
-            if hasattr(self.func, "__qualname__"):
-                func = self.func.__qualname__  # type: Optional[str]
-            elif hasattr(self.func, "__name__"):
-                func = self.func.func_name  # type: ignore[attr-defined]
-                if hasattr(self.func, "im_class"):
-                    func = self.func.im_class.__name__ + "." + func  # type: ignore[attr-defined]
-            else:
+            func = getattr(self.func, "__qualname__", None)
+            if func is None:
+                func = getattr(self.func, "__name__", None)
+                if func is not None:
+                    imClass = getattr(self.func, "im_class", None)
+                    if imClass is not None:
+                        func = f"{imClass}.{func}"
+            if func is None:
                 func = reflect.safe_repr(self.func)
         else:
             func = None
@@ -262,15 +263,12 @@ class DelayedCall:
             if self.kw:
                 L.append(
                     ", ".join(
-                        [
-                            "{}={}".format(k, reflect.safe_repr(v))
-                            for (k, v) in self.kw.items()
-                        ]
+                        [f"{k}={reflect.safe_repr(v)}" for (k, v) in self.kw.items()]
                     )
                 )
             L.append(")")
 
-        if self.debug:
+        if self.creator is not None:
             L.append("\n\ntraceback at creation: \n\n%s" % ("    ".join(self.creator)))
         L.append(">")
 
@@ -292,21 +290,21 @@ class ThreadedResolver:
 
     def __init__(self, reactor: "ReactorBase") -> None:
         self.reactor = reactor
-        self._runningQueries = {}  # type: Dict[Deferred, Tuple[Deferred, IDelayedCall]]
+        self._runningQueries: Dict[
+            Deferred[str], Tuple[Deferred[str], IDelayedCall]
+        ] = {}
 
     def _fail(self, name: str, err: str) -> Failure:
-        lookupError = error.DNSLookupError(
-            "address {!r} not found: {}".format(name, err)
-        )
+        lookupError = error.DNSLookupError(f"address {name!r} not found: {err}")
         return Failure(lookupError)
 
-    def _cleanup(self, name: str, lookupDeferred: Deferred) -> None:
+    def _cleanup(self, name: str, lookupDeferred: Deferred[str]) -> None:
         userDeferred, cancelCall = self._runningQueries[lookupDeferred]
         del self._runningQueries[lookupDeferred]
         userDeferred.errback(self._fail(name, "timeout error"))
 
     def _checkTimeout(
-        self, result: object, name: str, lookupDeferred: Deferred
+        self, result: str, name: str, lookupDeferred: Deferred[str]
     ) -> None:
         try:
             userDeferred, cancelCall = self._runningQueries[lookupDeferred]
@@ -323,7 +321,7 @@ class ThreadedResolver:
 
     def getHostByName(
         self, name: str, timeout: Sequence[int] = (1, 3, 11, 45)
-    ) -> Deferred:
+    ) -> Deferred[str]:
         """
         See L{twisted.internet.interfaces.IResolverSimple.getHostByName}.
 
@@ -335,7 +333,7 @@ class ThreadedResolver:
             timeoutDelay = sum(timeout)
         else:
             timeoutDelay = 60
-        userDeferred = defer.Deferred()
+        userDeferred: Deferred[str] = Deferred()
         lookupDeferred = threads.deferToThreadPool(
             self.reactor,
             cast(IReactorThreads, self.reactor).getThreadPool(),
@@ -354,11 +352,11 @@ class ThreadedResolver:
 class BlockingResolver:
     def getHostByName(
         self, name: str, timeout: Sequence[int] = (1, 3, 11, 45)
-    ) -> Deferred:
+    ) -> Deferred[str]:
         try:
             address = socket.gethostbyname(name)
         except OSError:
-            msg = "address {!r} not found".format(name)
+            msg = f"address {name!r} not found"
             err = error.DNSLookupError(msg)
             return defer.fail(err)
         else:
@@ -406,9 +404,9 @@ class _ThreePhaseEvent:
     """
 
     def __init__(self) -> None:
-        self.before = []  # type: List[_ThreePhaseEventTrigger]
-        self.during = []  # type: List[_ThreePhaseEventTrigger]
-        self.after = []  # type: List[_ThreePhaseEventTrigger]
+        self.before: List[_ThreePhaseEventTrigger] = []
+        self.during: List[_ThreePhaseEventTrigger] = []
+        self.after: List[_ThreePhaseEventTrigger] = []
         self.state = "BASE"
 
     def addTrigger(
@@ -416,7 +414,7 @@ class _ThreePhaseEvent:
         phase: str,
         callable: _ThreePhaseEventTriggerCallable,
         *args: object,
-        **kwargs: object
+        **kwargs: object,
     ) -> _ThreePhaseEventTriggerHandle:
         """
         Add a trigger to the indicate phase.
@@ -489,7 +487,7 @@ class _ThreePhaseEvent:
         """
         self.state = "BEFORE"
         self.finishedBefore = []
-        beforeResults = []  # type: List[object]
+        beforeResults: List[Deferred[object]] = []
         while self.before:
             callable, args, kwargs = self.before.pop(0)
             self.finishedBefore.append((callable, args, kwargs))
@@ -526,8 +524,8 @@ class PluggableResolverMixin:
     @ivar _nameResolver: The installed L{IHostnameResolver}.
     """
 
-    resolver = BlockingResolver()  # type: IResolverSimple
-    _nameResolver = _SimpleResolverComplexifier(resolver)  # type: IHostnameResolver
+    resolver: IResolverSimple = BlockingResolver()
+    _nameResolver: IHostnameResolver = _SimpleResolverComplexifier(resolver)
 
     # IReactorPluggableResolver
     def installResolver(self, resolver: IResolverSimple) -> IResolverSimple:
@@ -606,10 +604,10 @@ class ReactorBase(PluggableResolverMixin):
 
     def __init__(self) -> None:
         super().__init__()
-        self.threadCallQueue = []  # type: List[_ThreadCall]
-        self._eventTriggers = {}  # type: Dict[str, _ThreePhaseEvent]
-        self._pendingTimedCalls = []  # type: List[DelayedCall]
-        self._newTimedCalls = []  # type: List[DelayedCall]
+        self.threadCallQueue: List[_ThreadCall] = []
+        self._eventTriggers: Dict[str, _ThreePhaseEvent] = {}
+        self._pendingTimedCalls: List[DelayedCall] = []
+        self._newTimedCalls: List[DelayedCall] = []
         self._cancellations = 0
         self.running = False
         self._started = False
@@ -617,8 +615,8 @@ class ReactorBase(PluggableResolverMixin):
         self._startedBefore = False
         # reactor internal readers, e.g. the waker.
         # Using Any as the type here… unable to find a suitable defined interface
-        self._internalReaders = set()  # type: Set[Any]
-        self.waker = None  # type: Any
+        self._internalReaders: Set[Any] = set()
+        self.waker: Any = None
 
         # Arrange for the running attribute to change to True at the right time
         # and let a subclass possibly do other things at that time (eg install
@@ -693,7 +691,9 @@ class ReactorBase(PluggableResolverMixin):
         )
 
     # IReactorCore
-    def resolve(self, name: str, timeout: Sequence[int] = (1, 3, 11, 45)) -> Deferred:
+    def resolve(
+        self, name: str, timeout: Sequence[int] = (1, 3, 11, 45)
+    ) -> Deferred[str]:
         """
         Return a Deferred that will resolve a hostname."""
         if not name:
@@ -787,12 +787,12 @@ class ReactorBase(PluggableResolverMixin):
         eventType: str,
         callable: Callable[..., Any],
         *args: object,
-        **kwargs: object
+        **kwargs: object,
     ) -> _SystemEventID:
         """
         See twisted.internet.interfaces.IReactorCore.addSystemEventTrigger.
         """
-        assert builtins.callable(callable), "{} is not callable".format(callable)
+        assert builtins.callable(callable), f"{callable} is not callable"
         if eventType not in self._eventTriggers:
             self._eventTriggers[eventType] = _ThreePhaseEvent()
         return _SystemEventID(
@@ -868,8 +868,8 @@ class ReactorBase(PluggableResolverMixin):
         """
         See twisted.internet.interfaces.IReactorTime.callLater.
         """
-        assert builtins.callable(callable), "{} is not callable".format(callable)
-        assert delay >= 0, "{} is not greater than or equal to 0 seconds".format(delay)
+        assert builtins.callable(callable), f"{callable} is not callable"
+        assert delay >= 0, f"{delay} is not greater than or equal to 0 seconds"
         delayedCall = DelayedCall(
             self.seconds() + delay,
             callable,
@@ -905,14 +905,9 @@ class ReactorBase(PluggableResolverMixin):
     def _cancelCallLater(self, delayedCall: DelayedCall) -> None:
         self._cancellations += 1
 
-    def getDelayedCalls(self) -> List[IDelayedCall]:
+    def getDelayedCalls(self) -> Sequence[IDelayedCall]:
         """
-        Return all the outstanding delayed calls in the system.
-        They are returned in no particular order.
-        This method is not efficient -- it is really only meant for
-        test cases.
-
-        @return: A list of outstanding delayed calls.
+        See L{twisted.internet.interfaces.IReactorTime.getDelayedCalls}
         """
         return [
             x
@@ -943,7 +938,7 @@ class ReactorBase(PluggableResolverMixin):
         if not self._pendingTimedCalls:
             return None
 
-        delay = self._pendingTimedCalls[0].time - cast(float, self.seconds())
+        delay = self._pendingTimedCalls[0].time - self.seconds()
 
         # Pick a somewhat arbitrary maximum possible value for the timeout.
         # This value is 2 ** 31 / 1000, which is the number of seconds which can
@@ -998,8 +993,8 @@ class ReactorBase(PluggableResolverMixin):
                 call.called = 1
                 call.func(*call.args, **call.kw)
             except BaseException:
-                log.deferr()
-                if hasattr(call, "creator"):
+                log.err()
+                if call.creator is not None:
                     e = "\n"
                     e += (
                         " C: previous exception occurred in "
@@ -1024,112 +1019,6 @@ class ReactorBase(PluggableResolverMixin):
             self._justStopped = False
             self.fireSystemEvent("shutdown")
 
-    # IReactorProcess
-
-    def _checkProcessArgs(
-        self, args: List[Union[bytes, str]], env: Optional[Mapping[AnyStr, AnyStr]]
-    ) -> Union[
-        Tuple[List[bytes], Optional[Dict[bytes, bytes]]],
-        Tuple[List[Union[bytes, str]], Optional[Mapping[AnyStr, AnyStr]]],
-    ]:
-        """
-        Check for valid arguments and environment to spawnProcess.
-
-        @return: A two element tuple giving values to use when creating the
-        process.  The first element of the tuple is a C{list} of C{bytes}
-        giving the values for argv of the child process.  The second element
-        of the tuple is either L{None} if C{env} was L{None} or a C{dict}
-        mapping C{bytes} environment keys to C{bytes} environment values.
-        """
-        # Any unicode string which Python would successfully implicitly
-        # encode to a byte string would have worked before these explicit
-        # checks were added.  Anything which would have failed with a
-        # UnicodeEncodeError during that implicit encoding step would have
-        # raised an exception in the child process and that would have been
-        # a pain in the butt to debug.
-        #
-        # So, we will explicitly attempt the same encoding which Python
-        # would implicitly do later.  If it fails, we will report an error
-        # without ever spawning a child process.  If it succeeds, we'll save
-        # the result so that Python doesn't need to do it implicitly later.
-        #
-        # -exarkun
-
-        # If any of the following environment variables:
-        #  - PYTHONUTF8
-        #  - PYTHONIOENCODING
-        #
-        # are set before the Python interpreter runs, they will affect the
-        # value of sys.stdout.encoding.
-
-        # In certain cases, such as a Windows GUI Application which has no
-        # console, sys.stdout is None.  In this case,
-        # just return the args and env unmodified.
-        if not sys.stdout:
-            return args, env
-
-        # If a client application patches sys.stdout so that encoding is not
-        # set properly, try to fall back to sys.__stdout__.encoding.
-        defaultEncoding = sys.stdout.encoding or sys.__stdout__.encoding
-        if not defaultEncoding:
-            raise ValueError("sys.stdout does not have a valid encoding")
-
-        # Common check function
-        def argChecker(arg: Union[bytes, str]) -> Optional[bytes]:
-            """
-            Return either L{bytes} or L{None}.  If the given value is not
-            allowable for some reason, L{None} is returned.  Otherwise, a
-            possibly different object which should be used in place of arg is
-            returned.  This forces unicode encoding to happen now, rather than
-            implicitly later.
-            """
-            if isinstance(arg, str):
-                try:
-                    arg = arg.encode(defaultEncoding)
-                except UnicodeEncodeError:
-                    return None
-            if isinstance(arg, bytes) and b"\0" not in arg:
-                return arg
-
-            return None
-
-        # Make a few tests to check input validity
-        if not isinstance(args, (tuple, list)):
-            raise TypeError("Arguments must be a tuple or list")
-
-        outputArgs = []
-        for arg in args:
-            _arg = argChecker(arg)
-            if _arg is None:
-                raise TypeError(
-                    "Arguments contain a non-string value: {!r}".format(arg)
-                )
-            else:
-                outputArgs.append(_arg)
-
-        outputEnv = None
-        if env is not None:
-            outputEnv = {}
-            for key, val in env.items():
-                _key = argChecker(key)
-                if _key is None:
-                    raise TypeError(
-                        "Environment contains a "
-                        "non-string key: {!r}, using encoding: {}".format(
-                            key, sys.stdout.encoding
-                        )
-                    )
-                _val = argChecker(val)
-                if _val is None:
-                    raise TypeError(
-                        "Environment contains a "
-                        "non-string value: {!r}, using encoding {}".format(
-                            val, sys.stdout.encoding
-                        )
-                    )
-                outputEnv[_key] = _val
-        return outputArgs, outputEnv
-
     # IReactorThreads
     if platform.supportsThreads():
         assert ThreadPool is not None
@@ -1144,14 +1033,17 @@ class ReactorBase(PluggableResolverMixin):
             self.installNameResolver(_GAIResolver(self, self.getThreadPool))
             self.usingThreads = True
 
-        def callFromThread(
+        # `IReactorFromThreads` defines the first named argument as
+        # `callable: Callable[..., Any]` but this defines it as `f`
+        # really both should be defined using py3.8 positional only
+        def callFromThread(  # type: ignore[override]
             self, f: Callable[..., Any], *args: object, **kwargs: object
         ) -> None:
             """
             See
             L{twisted.internet.interfaces.IReactorFromThreads.callFromThread}.
             """
-            assert callable(f), "{} is not callable".format(f)
+            assert callable(f), f"{f} is not callable"
             # lists are thread-safe in CPython, but not in Jython
             # this is probably a bug in Jython, but until fixed this code
             # won't work in Jython.
@@ -1199,7 +1091,10 @@ class ReactorBase(PluggableResolverMixin):
                 assert self.threadpool is not None
             return self.threadpool
 
-        def callInThread(
+        # `IReactorInThreads` defines the first named argument as
+        # `callable: Callable[..., Any]` but this defines it as `_callable`
+        # really both should be defined using py3.8 positional only
+        def callInThread(  # type: ignore[override]
             self, _callable: Callable[..., Any], *args: object, **kwargs: object
         ) -> None:
             """
@@ -1218,7 +1113,7 @@ class ReactorBase(PluggableResolverMixin):
         def callFromThread(
             self, f: Callable[..., Any], *args: object, **kwargs: object
         ) -> None:
-            assert callable(f), "{} is not callable".format(f)
+            assert callable(f), f"{f} is not callable"
             # See comment in the other callFromThread implementation.
             self.threadCallQueue.append((f, args, kwargs))
 
@@ -1267,7 +1162,7 @@ class BaseConnector(ABC):
         if not self.factoryStarted:
             self.factory.doStart()
             self.factoryStarted = 1
-        self.transport = self._makeTransport()  # type: Optional[Client]
+        self.transport: Optional[Client] = self._makeTransport()
         if self.timeout is not None:
             self.timeoutID = self.reactor.callLater(
                 self.timeout, self.transport.failIfNotConnected, error.TimeoutError()
@@ -1292,7 +1187,7 @@ class BaseConnector(ABC):
                 pass
             del self.timeoutID
 
-    def buildProtocol(self, addr: Tuple[str, int]) -> IProtocol:
+    def buildProtocol(self, addr: IAddress) -> Optional[IProtocol]:
         self.state = "connected"
         self.cancelTimeout()
         return self.factory.buildProtocol(addr)
@@ -1335,8 +1230,8 @@ class BasePort(abstract.FileDescriptor):
     Note: This does not actually implement IListeningPort.
     """
 
-    addressFamily = None  # type: socket.AddressFamily
-    socketType = None  # type: socket.SocketKind
+    addressFamily: socket.AddressFamily = None  # type: ignore[assignment]
+    socketType: socket.SocketKind = None  # type: ignore[assignment]
 
     def createInternetSocket(self) -> socket.socket:
         s = socket.socket(self.addressFamily, self.socketType)
@@ -1438,7 +1333,7 @@ class _SignalReactorMixin:
                 log.msg("Unexpected error in main loop.")
                 log.err()
             else:
-                log.msg("Main loop terminated.")
+                log.msg("Main loop terminated.")  # type:ignore[unreachable]
 
 
-__all__ = []  # type: List[str]
+__all__: List[str] = []
