@@ -54,14 +54,13 @@ APIs listed above.
 """
 
 
-# System imports
-from OpenSSL import SSL  # type: ignore[import]
+from zope.interface import implementedBy, implementer, implementer_only
 
-from zope.interface import implementer, implementer_only, implementedBy
+# System imports
+from OpenSSL import SSL
 
 # Twisted imports
-from twisted.internet import tcp, interfaces
-
+from twisted.internet import interfaces, tcp
 
 supported = True
 
@@ -93,7 +92,7 @@ class DefaultOpenSSLContextFactory(ContextFactory):
         self,
         privateKeyFileName,
         certificateFileName,
-        sslmethod=SSL.SSLv23_METHOD,
+        sslmethod=SSL.TLS_METHOD,
         _contextFactory=SSL.Context,
     ):
         """
@@ -142,22 +141,22 @@ class ClientContextFactory:
 
     isClient = 1
 
-    # SSLv23_METHOD allows SSLv2, SSLv3, and TLSv1.  We disable SSLv2 below,
-    # though.
-    method = SSL.SSLv23_METHOD
+    # TLS_METHOD allows negotiation of multiple TLS versions.
+    method = SSL.TLS_METHOD
 
     _contextFactory = SSL.Context
 
     def getContext(self):
         ctx = self._contextFactory(self.method)
-        # See comment in DefaultOpenSSLContextFactory about SSLv2.
-        ctx.set_options(SSL.OP_NO_SSLv2)
+        ctx.set_options(
+            SSL.OP_NO_SSLv2 | SSL.OP_NO_SSLv3 | SSL.OP_NO_TLSv1 | SSL.OP_NO_TLSv1_1
+        )
         return ctx
 
 
 @implementer_only(
     interfaces.ISSLTransport,
-    *[i for i in implementedBy(tcp.Client) if i != interfaces.ITLSTransport],
+    *(i for i in implementedBy(tcp.Client) if i != interfaces.ITLSTransport),
 )
 class Client(tcp.Client):
     """
@@ -236,23 +235,23 @@ class Connector(tcp.Connector):
 
 
 from twisted.internet._sslverify import (
-    KeyPair,
-    DistinguishedName,
     DN,
     Certificate,
     CertificateRequest,
-    PrivateCertificate,
+    DistinguishedName,
+    KeyPair,
     OpenSSLAcceptableCiphers as AcceptableCiphers,
     OpenSSLCertificateOptions as CertificateOptions,
-    OpenSSLDiffieHellmanParameters as DiffieHellmanParameters,
-    platformTrust,
     OpenSSLDefaultPaths,
+    OpenSSLDiffieHellmanParameters as DiffieHellmanParameters,
+    PrivateCertificate,
+    ProtocolNegotiationSupport,
+    TLSVersion,
     VerificationError,
     optionsForClientTLS,
-    ProtocolNegotiationSupport,
+    platformTrust,
     protocolNegotiationMechanisms,
     trustRootFromCertificates,
-    TLSVersion,
 )
 
 __all__ = [
