@@ -5,23 +5,24 @@
 Tests for large portions of L{twisted.mail}.
 """
 
-import io
-import os
-import sys
-import textwrap
-import errno
-import shutil
-import pickle
 import email.message
 import email.parser
-import tempfile
+import errno
+import glob
+import io
+import os
+import pickle
+import shutil
 import signal
+import sys
+import tempfile
+import textwrap
 import time
 from hashlib import md5
 from unittest import skipIf
 
-from zope.interface.verify import verifyClass
 from zope.interface import Interface, implementer
+from zope.interface.verify import verifyClass
 
 import twisted.cred.checkers
 import twisted.cred.credentials
@@ -32,20 +33,19 @@ import twisted.mail.maildir
 import twisted.mail.protocols
 import twisted.mail.relay
 import twisted.mail.relaymanager
-
 from twisted import cred, mail
 from twisted.internet import address, defer, interfaces, protocol, reactor, task
 from twisted.internet.defer import Deferred
 from twisted.internet.error import (
-    DNSLookupError,
     CannotListenError,
+    DNSLookupError,
     ProcessDone,
     ProcessTerminated,
 )
 from twisted.mail import pop3, smtp
 from twisted.mail.relaymanager import _AttemptManager
 from twisted.names import dns
-from twisted.names.dns import RRHeader, Record_CNAME, Record_MX
+from twisted.names.dns import Record_CNAME, Record_MX, RRHeader
 from twisted.names.error import DNSNameError
 from twisted.python import failure, log
 from twisted.python.filepath import FilePath
@@ -221,25 +221,12 @@ class BounceWithSMTPServerTests(TestCase):
 @skipIf(platformType != "posix", "twisted.mail only works on posix")
 class FileMessageTests(TestCase):
     def setUp(self):
-        self.name = "fileMessage.testFile"
-        self.final = "final.fileMessage.testFile"
-        self.f = open(self.name, "w")
+        self.name = self.mktemp()
+        self.final = self.mktemp()
+        self.f = open(self.name, "wb")
+        self.addCleanup(self.f.close)
         self.fp = mail.mail.FileMessage(self.f, self.name, self.final)
 
-    def tearDown(self):
-        try:
-            self.f.close()
-        except BaseException:
-            pass
-        try:
-            os.remove(self.name)
-        except BaseException:
-            pass
-        try:
-            os.remove(self.final)
-        except BaseException:
-            pass
-
     def testFinalName(self):
         return self.fp.eomReceived().addCallback(self._cbFinalName)
 
@@ -248,18 +235,16 @@ class FileMessageTests(TestCase):
         self.assertTrue(self.f.closed)
         self.assertFalse(os.path.exists(self.name))
 
-    @skipIf(sys.version_info >= (3,), "not ported to Python 3")
     def testContents(self):
-        contents = "first line\nsecond line\nthird line\n"
+        contents = b"first line\nsecond line\nthird line\n"
         for line in contents.splitlines():
             self.fp.lineReceived(line)
         self.fp.eomReceived()
-        with open(self.final) as f:
+        with open(self.final, "rb") as f:
             self.assertEqual(f.read(), contents)
 
-    @skipIf(sys.version_info >= (3,), "not ported to Python 3")
     def testInterrupted(self):
-        contents = "first line\nsecond line\n"
+        contents = b"first line\nsecond line\n"
         for line in contents.splitlines():
             self.fp.lineReceived(line)
         self.fp.connectionLost()
@@ -267,6 +252,71 @@ class FileMessageTests(TestCase):
         self.assertFalse(os.path.exists(self.final))
 
 
+@skipIf(platformType != "posix", "twisted.mail only works on posix")
+class MaildirMessageTests(TestCase):
+    """
+    Tests for the file creating by the L{mail.maildir.MaildirMessage}.
+    """
+
+    def setUp(self):
+        """
+        Create and open a temporary file.
+        """
+        self.name = self.mktemp()
+        self.final = self.mktemp()
+        self.address = b"user@example.com"
+        self.f = open(self.name, "wb")
+        self.addCleanup(self.f.close)
+        self.fp = mail.maildir.MaildirMessage(
+            self.address, self.f, self.name, self.final
+        )
+
+    def _finalName(self):
+        """
+        Search for the final file path.
+
+        @rtype: L{str}
+        @return: Final file path.
+        """
+        return glob.glob(f"{self.final},S=[0-9]*")[0]
+
+    def test_finalName(self):
+        """
+        Send the EOM to the message and check that the final file name contains
+        the correct file size and the temporary file has been closed and removed.
+        """
+        final = self.successResultOf(self.fp.eomReceived())
+        self.assertEqual(final, f"{self.final},S={os.path.getsize(final)}")
+        self.assertTrue(self.f.closed)
+        self.assertFalse(os.path.exists(self.name))
+
+    def test_contents(self):
+        """
+        Send a message contents and the EOM to the message and check that the
+        final file contains the correct header and the message contents.
+        """
+        contents = b"first line\nsecond line\nthird line\n"
+        for line in contents.splitlines():
+            self.fp.lineReceived(line)
+        final = self.successResultOf(self.fp.eomReceived())
+        with open(final, "rb") as f:
+            self.assertEqual(
+                f.read(), b"Delivered-To: %s\n%s" % (self.address, contents)
+            )
+
+    def test_interrupted(self):
+        """
+        Check that the interrupted message transfer removes the temporary file
+        and a doesn't create a final file.
+        """
+        contents = b"first line\nsecond line\n"
+        for line in contents.splitlines():
+            self.fp.lineReceived(line)
+        self.fp.connectionLost()
+        self.assertFalse(os.path.exists(self.name))
+        self.assertRaises(IndexError, self._finalName)
+
+
 @skipIf(platformType != "posix", "twisted.mail only works on posix")
 class MailServiceTests(TestCase):
     def setUp(self):
@@ -1159,9 +1209,7 @@ class DirectoryQueueTests(TestCase):
             self.assertEqual(envelopes.pop(0), ["header", i])
 
 
-from twisted.names import server
-from twisted.names import client
-from twisted.names import common
+from twisted.names import client, common, server
 
 
 class TestAuthority(common.ResolverBase):
@@ -2352,7 +2400,7 @@ class DummyQueue:
         @return: The envelope file and a message receiver for a new message in
             the queue.
         """
-        fname = "{}_{}".format(time.time(), id(self))
+        fname = f"{time.time()}_{id(self)}"
         headerFile = open(os.path.join(self.directory, fname + "-H"), "wb")
         tempFilename = os.path.join(self.directory, fname + "-C")
         finalFilename = os.path.join(self.directory, fname + "-D")
