@@ -14,22 +14,17 @@ import inspect
 import random
 import socket
 import struct
-from itertools import chain
-
 from io import BytesIO
+from itertools import chain
 from typing import Optional, SupportsInt, Union
 
-from zope.interface import implementer, Interface, Attribute
-
+from zope.interface import Attribute, Interface, implementer
 
 # Twisted imports
-from twisted.internet import protocol, defer
+from twisted.internet import defer, protocol
 from twisted.internet.error import CannotListenError
-from twisted.python import log, failure
-from twisted.python import util as tputil
-from twisted.python import randbytes
-from twisted.python.compat import comparable, cmp, nativeString
-
+from twisted.python import failure, log, randbytes, util as tputil
+from twisted.python.compat import cmp, comparable, nativeString
 
 __all__ = [
     "IEncodable",
@@ -293,8 +288,11 @@ class IRecord(Interface):
 
 # Backwards compatibility aliases - these should be deprecated or something I
 # suppose. -exarkun
-from twisted.names.error import DomainError, AuthoritativeDomainError
-from twisted.names.error import DNSQueryTimeoutError
+from twisted.names.error import (
+    AuthoritativeDomainError,
+    DNSQueryTimeoutError,
+    DomainError,
+)
 
 
 def _nameToLabels(name):
@@ -344,7 +342,7 @@ def domainString(domain):
     if not isinstance(domain, bytes):
         raise TypeError(
             "Expected {} or {} but found {!r} of type {}".format(
-                type(b"").__name__, type("").__name__, domain, type(domain)
+                bytes.__name__, str.__name__, domain, type(domain)
             )
         )
     return domain
@@ -478,7 +476,7 @@ class IEncodableRecord(IEncodable, IRecord):
 class Charstr:
     def __init__(self, string=b""):
         if not isinstance(string, bytes):
-            raise ValueError("{!r} is not a byte string".format(string))
+            raise ValueError(f"{string!r} is not a byte string")
         self.string = string
 
     def encode(self, strio, compDict=None):
@@ -683,10 +681,10 @@ class Query:
             self.type, EXT_QUERIES.get(self.type, "UNKNOWN (%d)" % self.type)
         )
         c = QUERY_CLASSES.get(self.cls, "UNKNOWN (%d)" % self.cls)
-        return "<Query {} {} {}>".format(self.name, t, c)
+        return f"<Query {self.name} {t} {c}>"
 
     def __repr__(self) -> str:
-        return "Query({!r}, {!r}, {!r})".format(self.name.name, self.type, self.cls)
+        return f"Query({self.name.name!r}, {self.type!r}, {self.cls!r})"
 
 
 @implementer(IEncodable)
@@ -1076,7 +1074,7 @@ class SimpleRecord(tputil.FancyStrMixin, tputil.FancyEqMixin):
     showAttributes = (("name", "name", "%s"), "ttl")
     compareAttributes = ("name", "ttl")
 
-    TYPE = None  # type: Optional[int]
+    TYPE: Optional[int] = None
     name = None
 
     def __init__(self, name=b"", ttl=None):
@@ -1243,7 +1241,7 @@ class Record_A(tputil.FancyEqMixin):
         return hash(self.address)
 
     def __str__(self) -> str:
-        return "<A address={} ttl={}>".format(self.dottedQuad(), self.ttl)
+        return f"<A address={self.dottedQuad()} ttl={self.ttl}>"
 
     __repr__ = __str__
 
@@ -2360,7 +2358,7 @@ def _getDisplayableArguments(obj, alwaysShow, fieldNames):
         defaultValue = signature.parameters[name].default
         fieldValue = getattr(obj, name, defaultValue)
         if (name in alwaysShow) or (fieldValue != defaultValue):
-            displayableArgs.append(" {}={!r}".format(name, fieldValue))
+            displayableArgs.append(f" {name}={fieldValue!r}")
 
     return displayableArgs
 
@@ -2410,7 +2408,7 @@ def _compactRepr(
     for name in sectionNames:
         section = getattr(obj, name, [])
         if section:
-            out.append(" {}={!r}".format(name, section))
+            out.append(f" {name}={section!r}")
 
     out.append(">")
 
@@ -3220,6 +3218,9 @@ class DNSDatagramProtocol(DNSMixin, protocol.DatagramProtocol):
         except EOFError:
             log.msg("Truncated packet (%d bytes) from %s" % (len(data), addr))
             return
+        except ValueError as ex:
+            log.msg(f"Invalid packet ({ex}) from {addr}")
+            return
         except BaseException:
             # Nothing should trigger this, but since we're potentially
             # invoking a lot of different decoding methods, we might as well
