@@ -11,6 +11,7 @@ from unittest import TestCase
 from hamcrest import assert_that, equal_to, has_length
 from hamcrest.core.matcher import Matcher
 
+from twisted.internet.defer import Deferred
 from twisted.test.iosim import connectedServerAndClient
 from twisted.trial._dist.worker import LocalWorkerAMP, WorkerProtocol
 from twisted.trial.reporter import TestResult
@@ -27,9 +28,8 @@ def run(case: SynchronousTestCase, target: TestCase) -> TestResult:
     """
     result = TestResult()
     worker, local, pump = connectedServerAndClient(LocalWorkerAMP, WorkerProtocol)
-    d = local.run(target, result)
-    pump.pump()
-    pump.pump()
+    d = Deferred.fromCoroutine(local.run(target, result))
+    pump.flush()
     assert_that(case.successResultOf(d), equal_to({"success": True}))
     return result
 
@@ -46,6 +46,18 @@ class WorkerReporterTests(SynchronousTestCase):
         """
         assert_that(run(self, target), matches_result(**expectations))
 
+    def test_outsideReportingContext(self) -> None:
+        """
+        L{WorkerReporter}'s implementation of test result methods raise
+        L{ValueError} when called outside of the
+        L{WorkerReporter.gatherReportingResults} context manager.
+        """
+        worker, local, pump = connectedServerAndClient(LocalWorkerAMP, WorkerProtocol)
+
+        case = sample.FooTest("test_foo")
+        with self.assertRaises(ValueError):
+            worker._result.addSuccess(case)
+
     def test_addSuccess(self) -> None:
         """
         L{WorkerReporter} propagates successes.
@@ -60,6 +72,26 @@ class WorkerReporterTests(SynchronousTestCase):
             erroneous.TestAsynchronousFail("test_exception"), errors=has_length(1)
         )
 
+    def test_addErrorGreaterThan64k(self) -> None:
+        """
+        L{WorkerReporter} propagates errors with large string representations.
+        """
+        self.assertTestRun(
+            erroneous.TestAsynchronousFail("test_exceptionGreaterThan64k"),
+            errors=has_length(1),
+        )
+
+    def test_addErrorGreaterThan64kEncoded(self) -> None:
+        """
+        L{WorkerReporter} propagates errors with a string representation that
+        is smaller than an implementation-specific limit but which encode to a
+        byte representation that exceeds this limit.
+        """
+        self.assertTestRun(
+            erroneous.TestAsynchronousFail("test_exceptionGreaterThan64kEncoded"),
+            errors=has_length(1),
+        )
+
     def test_addErrorTuple(self) -> None:
         """
         L{WorkerReporter} propagates errors from pyunit's TestCases.
@@ -74,6 +106,15 @@ class WorkerReporterTests(SynchronousTestCase):
             erroneous.TestRegularFail("test_fail"), failures=has_length(1)
         )
 
+    def test_addFailureGreaterThan64k(self) -> None:
+        """
+        L{WorkerReporter} propagates test failures with large string representations.
+        """
+        self.assertTestRun(
+            erroneous.TestAsynchronousFail("test_failGreaterThan64k"),
+            failures=has_length(1),
+        )
+
     def test_addFailureTuple(self) -> None:
         """
         L{WorkerReporter} propagates test failures from pyunit's TestCases.
@@ -105,6 +146,15 @@ class WorkerReporterTests(SynchronousTestCase):
             skipping.SynchronousStrictTodo("test_todo1"), expectedFailures=has_length(1)
         )
 
+    def test_addExpectedFailureGreaterThan64k(self) -> None:
+        """
+        WorkerReporter propagates expected failures with large string representations.
+        """
+        self.assertTestRun(
+            skipping.ExpectedFailure("test_expectedFailureGreaterThan64k"),
+            expectedFailures=has_length(1),
+        )
+
     def test_addUnexpectedSuccess(self) -> None:
         """
         L{WorkerReporter} propagates unexpected successes.
